#ifndef	___SHINTAIAP_H_
#define	___SHINTAIAP_H_
//------------------------------------------------------------------------------
// BeOS
#include <InterfaceDefs.h>
#include <Menu.h>
#include <View.h>
// C++
#include <list>
#include <map>
#include <string>
// Add2
#include "GUIBase.h"
#include "Thread.h"
//------------------------------------------------------------------------------
const char	MSG_NAME_IAP_SOURCE_APP[] = "IAP:SrcApp";
const char	MSG_NAME_IAP_SOURCE_IAP_ITEM[] = "IAP:SrcIAPItem";
//------------------------------------------------------------------------------
const char	LKK_IAP_WIN_TITLE[] = "SHINTA Common Preferences";
const char	LKK_IAP_ACC_KEY_FORMAT[] = "Accelerator Format: ";
const char	LKK_IAP_ACC_KEY_NONE[] = "(None)";
const char	LKK_IAP_FLAT_TOOL_BAR[] = "Flat Tool Bar";
const char	LKK_IAP_SHOUSEI_USER_NAME[] = "Shousei User Name (Your Name): ";
const char	LKK_IAP_SHOUSEI_USER_NUMBER[] = "Shousei User Number (If you have): ";
const char	LKK_IAP_BUTTON_CANCEL[] = "Cancel";
const char	LKK_IAP_BUTTON_DEFAULTS[] = "Defaults";
const char	LKK_IAP_BUTTON_OK[] = "OK";
//------------------------------------------------------------------------------
const char	IAP_NAME_ACC_KEY_FORMAT[] = "AccKeyFormat2";
const char	IAP_NAME_FLAT_TOOL_BAR[] = "FlatToolBar";
const char	IAP_NAME_SHOUSEI_USER_NAME[] = "ShouseiUserName";
const char	IAP_NAME_SHOUSEI_USER_NUMBER[] = "ShouseiUserNumber";
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
LoadXXXX() は、ロードできなかったらデフォルト値を返す
派生クラスでは：
SetValue() は、oArc の中に適切な値が無かったら失敗するように実装すること
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TIAPItem
{
public:
	TIAPItem(const string& oName);
	virtual	~TIAPItem();

	virtual status_t	MakeConfigurationView(BView** oOutView, bool oDefault = false);
	string				Name() const;
	status_t			SetAppSignatureRef(const char* oAppSignatureRef);

	virtual status_t	GetValue(BMessage* oArc, bool oDefault = false) const;
	virtual status_t	SetValue(const BMessage& oArc);

	virtual status_t	GetBool(bool* oValue, bool oDefault = false) const;
	virtual status_t	SetBool(bool oValue);

	virtual status_t	GetString(string* oValue, bool oDefault = false) const;
	virtual status_t	SetString(const string& oValue);

protected:
	void				AddDefaultInfo(BMessage* oArc) const;
	const char*			AppSignatureRef() const;

	status_t			LoadBool(bool* oBool) const;
	status_t			SaveBool(bool oBool) const;

	status_t			LoadString(string* oString) const;
	status_t			SaveString(const string& oString) const;

private:
	status_t			Load(BMessage* oArc) const;
	status_t			Save(BMessage oArc) const;

	const char*	mAppSignatureRef;
	string		mName;
};
//==============================================================================
class	TIAPItemAccKeyFormat : public TIAPItem
{
public:
	TIAPItemAccKeyFormat();
	virtual	~TIAPItemAccKeyFormat();

	virtual status_t	MakeConfigurationView(BView** oOutView, bool oDefault = false);

	virtual status_t	GetValue(BMessage* oArc, bool oDefault = false) const;
	virtual status_t	SetValue(const BMessage& oArc);

	virtual status_t	GetString(string* oValue, bool oDefault = false) const;
	virtual status_t	SetString(const string& oValue);

private:
	typedef	TIAPItem	SUPER;

	void		AddMItem(BMenu* oMenu, const char* oString, bool oDefault) const;	// メニューアイテム追加

	string		mAccKeyFormat;		// アクセラレータキーの形式
};
//==============================================================================
class	TIAPItemFlatToolBar : public TIAPItem
{
public:
	TIAPItemFlatToolBar();
	virtual	~TIAPItemFlatToolBar();

	virtual status_t	MakeConfigurationView(BView** oOutView, bool oDefault = false);

	virtual status_t	GetValue(BMessage* oArc, bool oDefault = false) const;
	virtual status_t	SetValue(const BMessage& oArc);

	virtual status_t	GetBool(bool* oValue, bool oDefault = false) const;
	virtual status_t	SetBool(bool oValue);

private:
	typedef	TIAPItem	SUPER;

	bool		mFlatToolBar;		// ツールバーをフラットにする
};
//==============================================================================
class	TIAPItemShouseiUserName : public TIAPItem
{
public:
	TIAPItemShouseiUserName();
	~TIAPItemShouseiUserName();

	virtual status_t	MakeConfigurationView(BView** oOutView, bool oDefault = false);

	virtual status_t	GetValue(BMessage* oArc, bool oDefault = false) const;
	virtual status_t	SetValue(const BMessage& oArc);

	virtual status_t	GetString(string* oValue, bool oDefault = false) const;
	virtual status_t	SetString(const string& oValue);

private:
	typedef	TIAPItem	SUPER;

	string		mUserName;		// 翔星ユーザー名
};
//==============================================================================
class	TIAPItemShouseiUserNumber : public TIAPItem
{
public:
	TIAPItemShouseiUserNumber();
	~TIAPItemShouseiUserNumber();

	virtual status_t	MakeConfigurationView(BView** oOutView, bool oDefault = false);

	virtual status_t	GetValue(BMessage* oArc, bool oDefault = false) const;
	virtual status_t	SetValue(const BMessage& oArc);

	virtual status_t	GetString(string* oValue, bool oDefault = false) const;
	virtual status_t	SetString(const string& oValue);

private:
	typedef	TIAPItem	SUPER;

	string		mUserNumber;		// 翔星ユーザー番号
};
//==============================================================================
/* ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
典型的な使い方：
他のアプリから飛んできた SHINTA_IAP_CHANGED を捕捉するのはアプリの役目
アプリが SetValue() を実行
観察者のウィンドウやビューが変更に応じた処理を行う
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''' */
class	TInterAppPrefs
{
public:
	TInterAppPrefs();
	~TInterAppPrefs();

	status_t	AddItem(TIAPItem* oItem);
	status_t	AddObserver(const BMessenger& oObserver);	// SetValue() の観察者を追加
	status_t	CloseWindow(bool oForce = false) const;		// oForce は、自アプリで表示したウィンドウでなくても閉じるかどうか
	status_t	SetAppSignatureRef(const char* oAppSignatureRef);
	status_t	SetValue(const BMessage& oArc);
	status_t	ShowWindow(BRect oFrame);

	status_t	GetBool(const char* oName, bool* oValue) const;
	status_t	SetBool(const char* oName, bool oValue);

	status_t	GetString(const char* oName, string* oValue) const;
	status_t	SetString(const char* oName, const string& oValue);

private:
	void		ActivateWindow(port_id oWinPort) const;
	status_t	AddComponents(port_id oWinPort);
	port_id		CreateAndShowWindow(BRect oFrame);
	void		NotifyToObservers(BMessage oMsg);
	status_t	SendMessage(port_id oWinPort, BMessage* oMsg) const;

	bool		mWinOwner;
	const char*	mAppSignatureRef;
	list<BMessenger>		mObservers;
	map<string, TIAPItem*>	mItemMap;
};
//==============================================================================
class	TIAPPortManager : public TThread
{
public:
	TIAPPortManager(BMessenger oRecieveMsger);
	~TIAPPortManager();

	port_id			Port() const;
	status_t		PreventBlocking() const;

private:
	typedef	TThread	SUPER;

	virtual int32	Execute();

	port_id		mPort;
	BMessenger	mReceiveMsger;
};
//==============================================================================
class	TIAPView : public TDialogView
{
public:
	TIAPView(BRect oFrame);
	~TIAPView();

	void			ButtonCancelClicked();

private:
	typedef	TDialogView	SUPER;

	BView*			AddComponent(BMessage oArc, float oTop);
	virtual void	AttachedToWindow();
	void			ButtonDefaultsClicked();
	void			ButtonOKClicked();
	void			IAPActivateWindowRqtd();
	void			IAPAddComponentsRqtd(BMessage* oMsg);
	void			IAPControlChanged(BMessage* oMsg);
	void			LanguageChanged();
	virtual void	MessageReceived(BMessage* oMsg);
	void			SetComponents(int32 oIdx);		// mComponents のコンポーネント状態に設定
	void			SetViewLabels();
	status_t		SendMessage(BMessage* oMsg);	// ＳＨＩＮＴＡアプリにメッセージを送信
	virtual void	ShortCutDown(int32 oChar, int32 oFunc);

	BButton*		mButtonCancel;
	BButton*		mButtonDefaults;
	BButton*		mButtonOK;
	BMessage		mComponents;
	BView*			mButtonContainer;
	BView*			mLastAddChild;
	TInterAppPrefs	mIAPs;
};
//==============================================================================
class	TIAPWindow : public TDialogWindow
{
public:
	TIAPWindow(BRect oFrame);
	~TIAPWindow();

	port_id			Port() const;
	virtual void	Show();

private:
	typedef	TDialogWindow	SUPER;

	virtual void	LanguageChanged();
	virtual bool	QuitRequested();
	void			SetWindowTitle();

	TIAPPortManager*	mPortManager;
	TIAPView*			mIAPView;
};
//------------------------------------------------------------------------------
//==============================================================================
#endif	// ___SHINTAIAP_H_
